/*
roulette Version 0.2.0 - Roulette for Curses
Copyright (C) 2005-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses.h>
#include <time.h>
#include <errno.h>
#include "rnd.h"

#define BASE     38.0
#define PROB     (1.0 / 38.0)
/* function */
#define TYPE     0
#define BET      1
/* type */
#define STRAIGHT  0
#define SPLIT     1
#define STREET    2
#define QUARTER   3
#define LINE5     4
#define LINE6     5
#define COLUMN    6
#define DOZEN     7
#define LOW       8
#define HIGH      9
#define BLACK    10
#define RED      11
#define ODD      12
#define EVEN     13

typedef struct glbfmt {
   int run;      /* 0 = quit, 1 = run */
   int reset;    /* 0 = no reset, 1 = reset */
   int enter;    /* 0 = no enter, 1 = enter */
   int func;     /* type, square, bet, etc. */
   int type;     /* straight, split, etc. */
   int sq;       /* square #         */
   int i;        /* index into split */
   int j;        /* index into qtr   */
   int street;   /* low # of street  */
   int line;     /* low # of line    */
   int col;      /* low # of column  */
   int doz;      /* low # of dozen   */
   int bets;     /* # of bets played */
   int split[128]; /* pairs    of adjacent squares */
   int qtr[128];   /* quarters of adjacent squares */
   unsigned char *sd;  /* random number seed       */
   double amt;       /* amount per bet  */
   double bal;       /* balance         */
   double win;       /* total winning   */
   double loss;      /* total loss      */
   double maxbal;    /* maximum balance */
   double minbal;    /* minimum balance */
   } glbfmt;

void initglb(glbfmt *glb)
   {
   glb->run    = 1;
   glb->reset  = 0;
   glb->enter  = 0;
   glb->func   = TYPE;
   glb->type   = STRAIGHT;
   glb->sq     = 1;
   glb->i      = 0;
   glb->j      = 0;
   glb->street = 1;
   glb->line   = 1;
   glb->col    = 1;
   glb->doz    = 1;
   glb->bets   = 0;
   glb->amt    = 1.00;
   glb->bal    = 10000.00;
   glb->maxbal = glb->bal;
   glb->minbal = glb->bal;
   glb->win    = 0.00;
   glb->loss   = 0.00;
   glb->sd = (unsigned char *) rndinit();
   if (glb->sd == NULL)
      {
      fprintf(stderr,"roulette: out of memory "
	 "allocating glb->sd\n");
      exit(1);
      } /* if out of mem */
   glb->split[0] = 1;
   glb->split[1] = 2;
   glb->split[2] = 1;
   glb->split[3] = 4;
   glb->split[4] = 2;
   glb->split[5] = 3;
   glb->split[6] = 2;
   glb->split[7] = 5;
   glb->split[8] = 3;
   glb->split[9] = 6;

   glb->split[10] = 4;
   glb->split[11] = 5;
   glb->split[12] = 4;
   glb->split[13] = 7;
   glb->split[14] = 5;
   glb->split[15] = 6;
   glb->split[16] = 5;
   glb->split[17] = 8;
   glb->split[18] = 6;
   glb->split[19] = 9;

   glb->split[20] = 7;
   glb->split[21] = 8;
   glb->split[22] = 7;
   glb->split[23] = 10;
   glb->split[24] = 8;
   glb->split[25] = 9;
   glb->split[26] = 8;
   glb->split[27] = 11;
   glb->split[28] = 9;
   glb->split[29] = 12;

   glb->split[30] = 10;
   glb->split[31] = 11;
   glb->split[32] = 10;
   glb->split[33] = 13;
   glb->split[34] = 11;
   glb->split[35] = 12;
   glb->split[36] = 11;
   glb->split[37] = 14;
   glb->split[38] = 12;
   glb->split[39] = 15;

   glb->split[40] = 13;
   glb->split[41] = 14;
   glb->split[42] = 13;
   glb->split[43] = 16;
   glb->split[44] = 14;
   glb->split[45] = 15;
   glb->split[46] = 14;
   glb->split[47] = 17;
   glb->split[48] = 15;
   glb->split[49] = 18;

   glb->split[50] = 16;
   glb->split[51] = 17;
   glb->split[52] = 16;
   glb->split[53] = 19;
   glb->split[54] = 17;
   glb->split[55] = 18;
   glb->split[56] = 17;
   glb->split[57] = 20;
   glb->split[58] = 18;
   glb->split[59] = 21;

   glb->split[60] = 19;
   glb->split[61] = 20;
   glb->split[62] = 19;
   glb->split[63] = 22;
   glb->split[64] = 20;
   glb->split[65] = 21;
   glb->split[66] = 20;
   glb->split[67] = 23;
   glb->split[68] = 21;
   glb->split[69] = 24;

   glb->split[70] = 22;
   glb->split[71] = 23;
   glb->split[72] = 22;
   glb->split[73] = 25;
   glb->split[74] = 23;
   glb->split[75] = 24;
   glb->split[76] = 23;
   glb->split[77] = 26;
   glb->split[78] = 24;
   glb->split[79] = 27;

   glb->split[80] = 25;
   glb->split[81] = 26;
   glb->split[82] = 25;
   glb->split[83] = 28;
   glb->split[84] = 26;
   glb->split[85] = 27;
   glb->split[86] = 26;
   glb->split[87] = 29;
   glb->split[88] = 27;
   glb->split[89] = 30;

   glb->split[90] = 28;
   glb->split[91] = 29;
   glb->split[92] = 28;
   glb->split[93] = 31;
   glb->split[94] = 29;
   glb->split[95] = 30;
   glb->split[96] = 29;
   glb->split[97] = 32;
   glb->split[98] = 30;
   glb->split[99] = 33;

   glb->split[100] = 31;
   glb->split[101] = 32;
   glb->split[102] = 31;
   glb->split[103] = 34;
   glb->split[104] = 32;
   glb->split[105] = 33;
   glb->split[106] = 32;
   glb->split[107] = 35;
   glb->split[108] = 33;
   glb->split[109] = 36;

   glb->split[100] = 34;
   glb->split[101] = 35;
   glb->split[102] = 35;
   glb->split[103] = 36;

   glb->split[104] = 0;

   glb->qtr[0] = 1;
   glb->qtr[1] = 2;
   glb->qtr[2] = 4;
   glb->qtr[3] = 5;

   glb->qtr[4] = 2;
   glb->qtr[5] = 3;
   glb->qtr[6] = 5;
   glb->qtr[7] = 6;

   glb->qtr[8] = 4;
   glb->qtr[9] = 5;
   glb->qtr[10] = 7;
   glb->qtr[11] = 8;

   glb->qtr[12] = 5;
   glb->qtr[13] = 6;
   glb->qtr[14] = 8;
   glb->qtr[15] = 9;

   glb->qtr[16] = 7;
   glb->qtr[17] = 8;
   glb->qtr[18] = 10;
   glb->qtr[19] = 11;

   glb->qtr[20] = 8;
   glb->qtr[21] = 9;
   glb->qtr[22] = 11;
   glb->qtr[23] = 12;

   glb->qtr[24] = 10;
   glb->qtr[25] = 11;
   glb->qtr[26] = 13;
   glb->qtr[27] = 14;

   glb->qtr[28] = 11;
   glb->qtr[29] = 12;
   glb->qtr[30] = 14;
   glb->qtr[31] = 15;

   glb->qtr[32] = 13;
   glb->qtr[33] = 14;
   glb->qtr[34] = 16;
   glb->qtr[35] = 17;

   glb->qtr[36] = 14;
   glb->qtr[37] = 15;
   glb->qtr[38] = 17;
   glb->qtr[39] = 18;

   glb->qtr[40] = 16;
   glb->qtr[41] = 17;
   glb->qtr[42] = 19;
   glb->qtr[43] = 20;

   glb->qtr[44] = 17;
   glb->qtr[45] = 18;
   glb->qtr[46] = 20;
   glb->qtr[47] = 21;

   glb->qtr[48] = 19;
   glb->qtr[49] = 20;
   glb->qtr[50] = 22;
   glb->qtr[51] = 23;

   glb->qtr[52] = 20;
   glb->qtr[53] = 21;
   glb->qtr[54] = 23;
   glb->qtr[55] = 24;

   glb->qtr[56] = 22;
   glb->qtr[57] = 23;
   glb->qtr[58] = 25;
   glb->qtr[59] = 26;

   glb->qtr[60] = 23;
   glb->qtr[61] = 24;
   glb->qtr[62] = 26;
   glb->qtr[63] = 27;

   glb->qtr[64] = 25;
   glb->qtr[65] = 26;
   glb->qtr[66] = 28;
   glb->qtr[67] = 29;

   glb->qtr[68] = 26;
   glb->qtr[69] = 27;
   glb->qtr[70] = 29;
   glb->qtr[71] = 30;

   glb->qtr[72] = 28;
   glb->qtr[73] = 29;
   glb->qtr[74] = 31;
   glb->qtr[75] = 32;

   glb->qtr[76] = 29;
   glb->qtr[77] = 30;
   glb->qtr[78] = 32;
   glb->qtr[79] = 33;

   glb->qtr[80] = 31;
   glb->qtr[81] = 32;
   glb->qtr[82] = 34;
   glb->qtr[83] = 35;

   glb->qtr[84] = 32;
   glb->qtr[85] = 33;
   glb->qtr[86] = 35;
   glb->qtr[87] = 36;

   glb->qtr[88] = 0;
   } /* initglb */

void initnc()
   {
   initscr();
   cbreak();
   noecho();
   nonl();
   intrflush(stdscr,FALSE);
   keypad(stdscr,TRUE);
   nodelay(stdscr,TRUE);
   curs_set(0);
   clear();
   refresh();
   } /* initnc */

void slp(int ticks)
   {
   int i;
   int rslt;
   char str[32];
   struct timespec request;
   struct timespec remain;
   if (ticks < 0 || ticks > 1000000)
      {
      fprintf(stderr,"roulette: slp: invalid "
	 "ticks %d\n", ticks);
      exit(1);
      } /* bad ticks */
   request.tv_sec  = 0;
   request.tv_nsec = 1;
   remain.tv_sec   = 0;
   remain.tv_nsec  = 0;
   i = ticks;
   while (i--)
      {
      rslt = nanosleep(&request,&remain);
      if (rslt < 0)
	 {
	 if (errno == EINTR)
	    strcpy(str,"EINTR");
	 else if (errno == EINVAL)
	    strcpy(str,"EINVAL");
	 else
	    strcpy(str,"unknown value");
	 fprintf(stderr,"req sec %u "
	    " req nsec %u "
	    " rem sec %u "
	    " rem nsec %u\n",
            (unsigned int) request.tv_sec,
            (unsigned int) request.tv_nsec,
	    (unsigned int) remain.tv_sec,
	    (unsigned int) remain.tv_nsec);
	 fprintf(stderr,"roulette: slp: error %d %s\n",
	    errno, str);
	 perror("Nanosleep error");
	 exit(1);
	 } /* if err */
      } /* ticks loop */
   } /* slp */

void escape(glbfmt *glb)
   {
   int ch;
   ch = getch();
   if (0 && ch >= 0)
      {
      char str[64];
      move(20,0);
      sprintf(str,"%08x %d %u", ch, ch, ch);
      addstr(str);
      clrtoeol();
      refresh();
      } /* if char received */
   if (ch == 0x1b || ch == 'q')
      {
      glb->run = 0;
      } /* if quit */
   else if (ch == 13)
      {
      glb->enter = 1;
      } /* if enter */
   else if (ch == 'r')
      {
      glb->reset = 1;
      } /* if reset */
   else if (ch == KEY_LEFT)
      {
      glb->type--;
      if (glb->type < 0) glb->type = EVEN;
      } /* left */
   else if (ch == KEY_RIGHT)
      {
      glb->type++;
      if (glb->type > EVEN) glb->type = STRAIGHT;
      } /* right */
   else if (ch == KEY_NPAGE)
      {
      glb->func++;
      if (glb->func > BET) glb->func = TYPE;
      } /* if page down */
   else if (ch == KEY_PPAGE)
      {
      glb->func--;
      if (glb->func < TYPE) glb->func = BET;
      } /* if page up */
   else if (ch == KEY_DOWN)
      {
      if (glb->func == TYPE)
	 {
	 switch (glb->type) {
	    case STRAIGHT:
	       if (glb->sq < 36) glb->sq++;
	       break;
	    case SPLIT:
	       if (glb->i < 102) glb->i += 2;
	       break;
	    case STREET:
	       if (glb->street < 34) glb->street += 3;
	       break;
	    case QUARTER:
	       if (glb->j < 84) glb->j += 4;
	       break;
	    case LINE5:
	       glb->line = 1;
	       break;
	    case LINE6:
	       if (glb->line < 31) glb->line += 3;
	       break;
	    case COLUMN:
	       if (glb->col  < 25) glb->col += 12;
	       break;
	    case DOZEN:
	       if (glb->doz  < 3)  glb->doz++;
	       break;
	    case LOW:
            case HIGH:
            case BLACK:
            case RED:
            case ODD:
            case EVEN:
	       break;
	    default:
               endwin();
	       fprintf(stderr,"Logic error 001\n");
	       exit(1);
	    } /* switch type */
	 } /* if set square */
      else if (glb->func == BET)
	 {
	 if (glb->type <= LINE6)
	    {
	    if (glb->amt <= 50.00
	       && glb->bal - 1.00 >= glb->amt * 2.0)
	       {
	       glb->amt *= 2.00;
	       } /* if within limits */
	    } /* if straight - line6 */
	 else if (glb->type <= DOZEN)
	    {
	    if (glb->amt <= 125.00
	       && glb->bal - 1.00 >= glb->amt * 2.0)
	       {
	       glb->amt *= 2.00;
	       } /* if within limits */
	    } /* if column or dozen */
	 else
	    {
	    if (glb->amt <= 250.00
	       && glb->bal - 1.00 >= glb->amt * 2.0)
	       {
	       glb->amt *= 2.00;
	       } /* if within limits */
	    } /* if color or e/o or h/l */
	 } /* if increase bet x 2 */
      } /* down */
   else if (ch == KEY_UP)
      {
      if (glb->func == TYPE)
	 {
	 switch (glb->type) {
	    case STRAIGHT:
	       if (glb->sq > 1) glb->sq--;
	       break;
	    case SPLIT:
	       if (glb->i >= 2) glb->i -= 2;
	       break;
	    case STREET:
	       if (glb->street >= 4) glb->street -= 3;
	       break;
	    case QUARTER:
	       if (glb->j >= 4) glb->j -= 4;
	       break;
	    case LINE5:
	       glb->line = 1;
	       break;
	    case LINE6:
	       if (glb->line >= 4) glb->line -= 3;
	       break;
	    case COLUMN:
	       if (glb->col > 12)  glb->col-=12;
	       break;
	    case DOZEN:
	       if (glb->doz > 1) glb->doz--;
	       break;
	    case LOW:
            case HIGH:
            case BLACK:
            case RED:
            case ODD:
            case EVEN:
	       break;
	    default:
               endwin();
	       fprintf(stderr,"Logic error 002\n");
	       exit(1);
	    } /* switch type */
	 } /* if set square */
      else if (glb->func == BET)
	 {
	 if (glb->amt >= 2.00)
	    {
	    glb->amt /= 2.00;
	    } /* if within limits */
	 } /* if decrease bet / 2 */
      } /* up */
   else if (ch == KEY_END)
      {
      if (glb->func == BET)
	 {
	 if (glb->type <= LINE6)
	    {
	    if (glb->amt < glb->bal - 1.00
	       && glb->bal >= 101.00)
	       {
	       glb->amt = 100.00;
	       } /* if balance over limit */
	    else
	       {
	       glb->amt = glb->bal;
	       } /* else balance within limit */
	    } /* if straight - line6 */
	 else if (glb->type <= DOZEN)
	    {
	    if (glb->amt < glb->bal - 1.00
	       && glb->bal >= 251.00)
	       {
	       glb->amt = 250.00;
	       } /* if balance over limit */
	    else
	       {
	       glb->amt = glb->bal;
	       } /* else balance within limit */
	    } /* if column or dozen */
	 else
	    {
	    if (glb->amt < glb->bal - 1.00
	       && glb->bal >= 501.00)
	       {
	       glb->amt = 500.00;
	       } /* if balance over limit */
	    else
	       {
	       glb->amt = glb->bal;
	       } /* else balance within limit */
	    } /* if color or e/o or h/l */
	 } /* if set bet to balance */
      } /* end */
   else if (ch == KEY_HOME)
      {
      if (glb->func == BET)
	 {
         glb->amt = 1.00;
	 } /* if set bet to 1 */
      } /* home */
   else if (ch == KEY_F0 + 1 || ch == 'u' || ch == 'U')
      {
      if (glb->func == BET)
	 {
	 if (glb->type <= LINE6)
	    {
	    if (glb->amt <= 99.00
	       && glb->bal > glb->amt + 2.0)
	       {
	       glb->amt += 1.00;
	       } /* if within limits */
	    } /* if straight - line6 */
	 else if (glb->type <= DOZEN)
	    {
	    if (glb->amt <= 249.00
	       && glb->bal > glb->amt + 2.0)
	       {
	       glb->amt += 1.00;
	       } /* if within limits */
	    } /* if column or dozen */
	 else
	    {
	    if (glb->amt <= 499.00
	       && glb->bal > glb->amt + 2.0)
	       {
	       glb->amt += 1.00;
	       } /* if within limits */
	    } /* if color or e/o or h/l */
	 } /* if increase bet +1 */
      } /* F1 */
   else if (ch == KEY_F0 + 2 || ch == 'd' || ch == 'D')
      {
      if (glb->func == BET)
	 {
         if (glb->amt >= 2.00)
            {
            glb->amt -= 1.00;
            } /* if within limits */
	 } /* if decrease bet -1 */
      } /* F2 */
   } /* escape */

void bldmenu(glbfmt *glb)
   {
   move(0,30);
   addstr("Roulette");
   move(2,2);
   addstr("Right = next type");
   move(3,2);
   addstr("Left  = prev type");
   move(4,2);
   addstr("Up    = lower  value");
   move(5,2);
   addstr("Down  = higher value");
   move(6,2);
   addstr("Home  = bet 1.00");
   move(7,2);
   addstr("End   = bet balance");
   move(8,2);
   addstr("F1    = bet +1");
   move(9,2);
   addstr("F2    = bet -1");
   move(10,2);
   addstr("PgDn  = next function");
   move(11,2);
   addstr("PgUp  = prev function");
   move(12,2);
   addstr("Enter = spin the wheel");
   move(13,2);
   addstr("r     = reset balance");
   move(14,2);
   addstr("q     = quit");
   refresh();
   } /* bldmenu */

void bldscr(glbfmt *glb)
   {
   char str[128];
   move(2,45);
   clrtoeol();
   if (glb->func == 0) addstr("Func type");
   else if (glb->func == 1) addstr("Func bet");
   move(3,45);
   clrtoeol();
   sprintf(str,"Type");
   addstr(str);
   switch(glb->type) {
      case STRAIGHT:
         sprintf(str," straight %2d", glb->sq);
         addstr(str);
	 break;
      case SPLIT:
         sprintf(str," split %2d %2d",
	    glb->split[glb->i],
	    glb->split[glb->i+1]);
         addstr(str);
	 break;
      case STREET:
         sprintf(str," street %2d", glb->street);
         addstr(str);
	 break;
      case QUARTER:
         sprintf(str," qtr %2d %2d %2d %2d",
	    glb->qtr[glb->j],
	    glb->qtr[glb->j+1],
	    glb->qtr[glb->j+2],
	    glb->qtr[glb->j+3]);
         addstr(str);
	 break;
      case LINE5:
         sprintf(str," line5 %2d", glb->line);
         addstr(str);
	 break;
      case LINE6:
         sprintf(str," line6 %2d", glb->line);
         addstr(str);
	 break;
      case COLUMN:
         sprintf(str," column %2d", glb->col);
         addstr(str);
	 break;
      case DOZEN:
         sprintf(str," dozen %2d", glb->doz);
         addstr(str);
	 break;
      case LOW:
         addstr(" low 1-18");
	 break;
      case HIGH:
         addstr(" high 19-36");
	 break;
      case BLACK:
         addstr(" black");
	 break;
      case RED:
         addstr(" red");
	 break;
      case ODD:
         addstr(" odd");
	 break;
      case EVEN:
         addstr(" even");
	 break;
      default:
         endwin();
	 fprintf(stderr,"Logic error 003\n");
	 exit(1);
      } /* switch type */
   move(4,45);
   clrtoeol();
   sprintf(str,"Spl Indx %3d", glb->i);
   addstr(str);
   move(5,45);
   clrtoeol();
   sprintf(str,"Qtr Indx %3d", glb->j);
   addstr(str);
   move(7,45);
   clrtoeol();
   sprintf(str,"Bet  %10.2f", glb->amt);
   addstr(str);
   move(8,45);
   clrtoeol();
   sprintf(str,"Bal  %10.2f", glb->bal);
   addstr(str);
   move(9,45);
   clrtoeol();
   sprintf(str,"Win  %10.2f", glb->win);
   addstr(str);
   move(10,45);
   clrtoeol();
   sprintf(str,"Loss %10.2f", glb->loss);
   addstr(str);
   move(11,45);
   clrtoeol();
   sprintf(str,"Max  %10.2f", glb->maxbal);
   addstr(str);
   move(12,45);
   clrtoeol();
   sprintf(str,"Min  %10.2f", glb->minbal);
   addstr(str);
   move(13,45);
   clrtoeol();
   sprintf(str,"Net  %10.2f", glb->win - glb->loss);
   addstr(str);
   move(14,45);
   clrtoeol();
   sprintf(str,"Bets %7d", glb->bets);
   addstr(str);
   refresh();
   } /* bldscr */

void play(glbfmt *glb)
   {
   int rndsq;
   int tmpsq;
   char str[64];
   glb->bets++;
   rndsq = (int) rndnum(38,glb->sd) + 1;
   move(15,45);
   clrtoeol();
   sprintf(str,"Spin %7d", rndsq);
   addstr(str);
   refresh();
   switch(glb->type) {
      case STRAIGHT:
         if (rndsq == glb->sq)
	    {
	    glb->win += (glb->amt * 35.0);
	    glb->bal += (glb->amt * 35.0);
	    } /* if bingo */
	 break;
      case SPLIT:
	 if (rndsq == glb->split[glb->i]
	    || rndsq == glb->split[glb->i+1])
	    {
	    glb->win += (glb->amt * 17.0);
	    glb->bal += (glb->amt * 17.0);
	    } /* if bingo */
	 break;
      case STREET:
         if (rndsq == glb->street
            || rndsq == glb->street+1
            || rndsq == glb->street+2)
	    {
	    glb->win += (glb->amt * 11.67);
	    glb->bal += (glb->amt * 11.67);
	    } /* if bingo */
	 break;
      case QUARTER:
	 if (rndsq == glb->split[glb->j]
	    || rndsq == glb->split[glb->j+1]
	    || rndsq == glb->split[glb->j+2]
	    || rndsq == glb->split[glb->j+3])
	    {
	    glb->win += (glb->amt * 8.00);
	    glb->bal += (glb->amt * 8.00);
	    } /* if bingo */
	 break;
      case LINE5:
         if (rndsq >= 1
            && rndsq <= 3)
	    {
	    glb->win += (glb->amt * 6.00);
	    glb->bal += (glb->amt * 6.00);
	    } /* if bingo */
         else if (rndsq == 37
            || rndsq == 38)
	    {
	    glb->win += (glb->amt * 6.00);
	    glb->bal += (glb->amt * 6.00);
	    } /* if bingo */
	 break;
      case LINE6:
         if (rndsq >= glb->line
            && rndsq <= glb->line+5)
	    {
	    glb->win += (glb->amt * 5.00);
	    glb->bal += (glb->amt * 5.00);
	    } /* if bingo */
	 break;
      case COLUMN:
	 tmpsq = ((rndsq - 1) / 12) + 1;
	 if (tmpsq == glb->col)
	    {
	    glb->win += (glb->amt * 2.00);
	    glb->bal += (glb->amt * 2.00);
	    } /* if bingo */
	 break;
      case DOZEN:
	 tmpsq = ((rndsq - 1) % 3) + 1;
	 if (tmpsq == glb->col)
	    {
	    glb->win += (glb->amt * 2.00);
	    glb->bal += (glb->amt * 2.00);
	    } /* if bingo */
	 break;
      case LOW:
	 tmpsq = (rndsq - 1) / 18;
	 if (!tmpsq)
	    {
	    glb->win += (glb->amt * 1.00);
	    glb->bal += (glb->amt * 1.00);
	    } /* if bingo */
	 break;
      case HIGH:
	 tmpsq = (rndsq - 1) / 18;
	 if (tmpsq == 1)
	    {
	    glb->win += (glb->amt * 1.00);
	    glb->bal += (glb->amt * 1.00);
	    } /* if bingo */
	 break;
      case BLACK:
	 switch(rndsq) {
	    case 2:
	    case 4:
	    case 6:
	    case 8:
	    case 10:
	    case 11:
	    case 13:
	    case 15:
	    case 17:
	    case 20:
	    case 22:
	    case 24:
	    case 26:
	    case 28:
	    case 29:
	    case 31:
	    case 33:
	    case 35:
	       glb->win += (glb->amt * 1.00);
	       glb->bal += (glb->amt * 1.00);
	       break;
	    } /* switch rndsq */
	 break;
      case RED:
	 switch(rndsq) {
	    case 1:
	    case 3:
	    case 5:
	    case 7:
	    case 9:
	    case 12:
	    case 14:
	    case 16:
	    case 18:
	    case 19:
	    case 21:
	    case 23:
	    case 25:
	    case 27:
	    case 30:
	    case 32:
	    case 34:
	    case 36:
	       glb->win += (glb->amt * 1.00);
	       glb->bal += (glb->amt * 1.00);
	       break;
	    } /* switch rndsq */
	 break;
      case ODD:
	 tmpsq = rndsq % 2;
	 if (tmpsq == 1)
	    {
	    glb->win += (glb->amt * 1.00);
	    glb->bal += (glb->amt * 1.00);
	    } /* if bingo */
	 break;
      case EVEN:
	 tmpsq = rndsq % 2;
	 if (!tmpsq)
	    {
	    glb->win += (glb->amt * 1.00);
	    glb->bal += (glb->amt * 1.00);
	    } /* if bingo */
	 break;
      default:
         endwin();
	 fprintf(stderr,"Logic error 004\n");
	 exit(1);
      } /* switch type */
   } /* play */

void run(glbfmt *glb)
   {
   while (glb->run)
      {
      bldscr(glb);
      escape(glb);
      if (glb->run && glb->reset)
	 {
	 clear();
	 bldmenu(glb);
	 initglb(glb);
	 glb->reset = 0;
	 } /* if reset */
      else if (glb->run && glb->enter)
	 {
	 if (glb->bal - 1.00 >= glb->amt)
	    {
	    glb->bal  -= glb->amt;
	    glb->loss += glb->amt;
	    if (glb->bal < glb->minbal)
	       glb->minbal = glb->bal;
	    move(20,0);
	    clrtoeol();
	    refresh();
	    play(glb);
	    if (glb->bal > glb->maxbal)
	       glb->maxbal = glb->bal;
	    glb->enter = 0;
	    } /* if enough balance to play */
	 else if (glb->bal - 1.00 < glb->amt)
	    {
	    if (glb->amt > 1.00)
	       {
	       move(20,20);
	       clrtoeol();
	       addstr("Out of money: reduce betting amount\n");
	       refresh();
	       glb->enter = 0;
	       } /* reduce betting amount??? */
	    else
	       {
               endwin();
	       fprintf(stderr,"roulette: "
	          "out of money\n");
	       fprintf(stderr,"The betting amount is "
	          "%12.2f\n", glb->amt);
	       fprintf(stderr,"Your balance is       "
	          "%12.2f\n", glb->bal);
	       fprintf(stderr,"Maximum balance       "
	          "%12.2f\n", glb->maxbal);
	       fprintf(stderr,"Minimum balance       "
	          "%12.2f\n", glb->minbal);
	       fprintf(stderr,"Total winning is      "
	          "%12.2f\n", glb->win);
	       fprintf(stderr,"Total loss is         "
	          "%12.2f\n", glb->loss);
	       fprintf(stderr,"Net win/loss is       "
	          "%12.2f\n", glb->win - glb->loss);
	       fprintf(stderr,"Total bets            "
	          "%9d\n", glb->bets);
	       fprintf(stderr,"The minimum "
	          "balance is 1.00.\n");
	       exit(0);
	       } /* no money left */
	    } /* if not enough balance to play */
	 } /* if enter */
      if (glb->run) slp(1);
      } /* display iteration */
   } /* runclk */

int main()
   {
   glbfmt *glb;
   glb = (glbfmt *) malloc(sizeof(glbfmt));
   if (glb == NULL)
      {
      fprintf(stderr,"roulette: out of memory "
	 "allocating glb\n");
      exit(1);
      } /* if out of mem */
   initglb(glb);
   initnc();
   bldmenu(glb);
   run(glb);
   endwin();
   printf("Ending balance   %12.2f\n",
      glb->bal);
   printf("Maximum balance  %12.2f\n",
      glb->maxbal);
   printf("Minimum balance  %12.2f\n",
      glb->minbal);
   printf("Total winning    %12.2f\n",
      glb->win);
   printf("Total loss       %12.2f\n",
      glb->loss);
   printf("Net win/loss     %12.2f\n",
      glb->win - glb->loss);
   printf("Total bets       %9d\n",
      glb->bets);
   free(glb->sd);
   free(glb);
   return(0);
   } /* main */
